@@ -1,6 +1,6 @@
-/* $Id: decode.c,v 1.12 2005/12/16 17:55:25 ichiro Exp $ */
+/* $Id: decode.c,v 1.36 2006/01/08 10:07:50 ichiro Exp $ */
 /*
- * Copyright (c) 2004
+ * Copyright (c) 2004, 2005, 2006
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
  * All rights reserved.
  *
@@ -31,6 +31,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -49,13 +50,64 @@
 
 #include "kircd.h"
 
-int url_decode_jis(char *dst, char *src)
-{
-	int n, len = 0;
-	char *p;
-	unsigned char data[HTTPBUF + 256];
+#ifdef HAVE_ICONV
+#include <iconv.h>
+#endif
 
-	p = dst;
+extern int debug;
+
+int charconv(char *dst, size_t dst_len, const char *src, size_t src_len, const char *to, const char *from)
+{
+	iconv_t ic;
+	int result = 0;
+	int i;
+	const char *p_in;
+	char *p_out;
+
+	p_in = src;
+	p_out = dst;
+
+	ic = iconv_open(to, from);
+
+	if (ic == (iconv_t)(-1)) {
+		printf("errno %d: iconv_open(%s, %s)\n", errno, to, from);
+		iconv_close(ic);
+		return -1;
+	}
+
+	/* iconv() */
+	DPRINTF(0, ("%s(%s) ->(%s)\n", src, from, to));
+	result = iconv(ic, &p_in, &src_len, &p_out, &dst_len);
+	if (result == (size_t)(-1))
+		printf("errno %d: iconv() %s -> %s\n", errno, from, to);
+
+	/* print src dst */
+	DPRINTF(1, ("iconv:[%s]\n<%s>(", src, from));
+	for (i = 0; i < strlen(src); *src++)
+		DPRINTF(1, ("%02x", src[i]));
+	DPRINTF(1, (") ->\n<%s>(", to));
+	for (i = 0; i < strlen(dst); *dst++)
+		DPRINTF(1, ("%02x", dst[i]));
+	DPRINTF(1, (")\n"));
+
+	iconv_close(ic);
+
+	return result;
+}
+
+int url_decode_jis(char *src)
+{
+	int n, j;
+	int len = 0;
+	int count = 0;
+	char *p;
+
+	const char *q;
+	size_t in_len, out_len;
+	unsigned char data[IRC_MAX * 2];
+	out_len = sizeof(data);
+
+	p = src;
 	bzero(data, sizeof(data));
 
 	if (strlen(src) == 0) 
@@ -63,44 +115,58 @@ int url_decode_jis(char *dst, char *src)
 
 	/* URL convert to raw data */
 	len = url_decode_rawdata(data, src);
+	q = data;
+	in_len = len + 1;
 
 	/* check encode method */
 	for (n = 0; n < len; n++) {
 		if (data[n] == 0x1B) { /* found JIScode */
-#ifdef DEBUG
-	printf("found JIS\n");
-#endif
-			/* printf("found JIS\n"); */
+			DPRINTF(1, ("found JIS\n"));
 			strcpy(p, data);
 			return (0);
 		}
 
+		count = 0;
+		if (data[n] >= 0xC0 && data[n] <= 0xF7) {
+			if (data[n] >= 0xC0 &&
+			    data[n] <= 0xDF) { /* UCS-2 */
+				count = 1;
+			} else if(data[n] >= 0xE0 &&
+				  data[n] <= 0xEF) { /* UCS-3 */
+				count = 2;
+			} else if(data[n] >= 0xF0 &&
+				  data[n] <= 0xF7) { /* UCS-4*/
+				count = 3;
+			}
+			for (j = 0; j < count; j++) {
+				if (data[n+1] < 0x80 ||
+				    data[n+1] > 0xBF)
+					goto OUT; /* not UTF-8 */
+			}
+			DPRINTF(1, ("found UTF-8\n"));
+			charconv(p, out_len, q, in_len, "iso-2022-jp", "utf-8");
+			return (0);
+		}
+OUT:
+
 		if (data[n] < 0x80) { 
 			goto loop_end;
 		} else if (data[n] >= 0x81 && data[n] <= 0x9F) {
-#ifdef DEBUG
-	printf("found SJIS\n");
-#endif
-			/* printf("found SJIS\n"); */
-			sjis2jis(p, data);
+			DPRINTF(1, ("found CP932\n"));
+			charconv(p, out_len, q, in_len, "iso-2022-jp", "cp932");
 			return (0);
 		} else if (data[n] >= 0xA1 && data[n] <= 0xDF) {
 			if ((data[n+1] < 0x80) ||
 			    (data[n+1] > 0x80 && data[n+1] < 0xA0)) {
-#ifdef DEBUG
-	printf("found SJIS+kana\n");
-#endif
-				/* printf("found SJIS+kana\n"); */
-				sjis2jis(p, data);
+				DPRINTF(1, ("found CP932\n"));
+				charconv(p, out_len, q, in_len, "iso-2022-jp", "cp932");
 				return (0);
 			}
 		}
 
 		if (data[n] >= 0xF0 && data[n] <= 0xFE) {
-#ifdef DEBUG
-	printf("found EUC\n");
-#endif
-			euc2jis(p, data);
+			DPRINTF(1, ("found EUC\n"));
+			charconv(p, out_len, q, in_len, "iso-2022-jp", "euc-jp");
 			return (0);
 		} else { 
 			n++;
@@ -108,12 +174,12 @@ int url_decode_jis(char *dst, char *src)
 
 		if ((data[n] >= 0x40 && data[n] <= 0x7E) || 
 		    (data[n] >= 0x80 && data[n] <= 0xA0)) {
-			/* printf("found SJIS\n"); */
-			sjis2jis(p, data);
+			DPRINTF(1, ("found CP932\n"));
+			charconv(p, out_len, q, in_len, "iso-2022-jp", "cp932");
 			return (0);
 		} else if (data[n] >= 0xFD && data[n] <= 0xFE) {
-			/* printf("found EUC\n"); */
-			euc2jis(p, data);
+			DPRINTF(1, ("found EUC\n"));
+			charconv(p, out_len, q, in_len, "iso-2022-jp", "euc-jp");
 			return (0);
 		}
 loop_end:
@@ -121,7 +187,35 @@ loop_end:
 	}
 
 	/* printf("use EUC conv\n"); */
-	euc2jis(p, data);
+	charconv(p, out_len, q, in_len, "iso-2022-jp", "euc-jp");
+
+	return (0);
+}
+
+int decode_jis2cp932(char *src)
+{
+	int n, i, len;
+	char *p;
+	const char *q;
+	size_t in_len, out_len;
+	unsigned char data[IRC_MAX * 2];
+	out_len = sizeof(data);
+
+	len = strlen(src);
+
+	strcpy(data, src);
+	p = src;
+	q = data;
+	in_len = len + 1;
+
+	/* check encode */
+	for (n = 0; n < len; n++) {
+		if (src[n] == 0x1B) { /* found JIScode */
+			DPRINTF(1, ("decode_jis2cp932()\n"));
+			charconv(p, out_len, q, in_len, "cp932", "iso-2022-jp");
+			return (0);
+		}
+	}
 
 	return (0);
 }
@@ -154,126 +248,6 @@ int url_decode_rawdata(char *dst, char *src)
 	return j;
 }
 
-int sjis2jis(char *dst, char *src)
-{
-	int i, j, len, esc_in;
-	char *p;
-	unsigned char high, low;
-
-	len = strlen((char *)src);
-	p = dst;
-	esc_in = 0;
-
-	for (i = 0, j = 0; i < len; i++, j++){
-		high = src[i];
-		low  = src[i+1];
-		if ((high >= 0x81 && high <= 0x9F) ||
-		     (high >= 0xE0 && high <= 0xEF)) {
-			if (high <= 0x9F) {
-				if(low < 0x9F)
-					high = (high << 1) - 0xE1;
-				else
-					high = (high << 1) - 0xE0;
-			} else {
-				if(low < 0x9F)
-					high = (high << 1) - 0x161;
-				else
-					high = (high << 1) - 0x160;
-			}
-			if (low < 0x7F)
-				low -= 0x1F;
-			else if (low < 0x9F)
-				low -= 0x20;
-			else
-				low -= 0x7E;
-
-			/* add ESC-IN code */
-			if (esc_in == 0) {
-			/* printf("1:ESC-IN[%d]\n", j); */
-				p[j] = 0x1B;
-				p[j+1] = 0x24;
-				p[j+2] = 0x42;
-				j += 3;
-				esc_in = 1;
-			}
-
-			/* JIS code into buffer */
-			p[j] = high;
-			p[j+1] = low;
-			/* printf("2:p[%d]=%x,p[%d]=%x\n",
-				j, p[j], j+1, p[j+1]); */
-			i++;
-			j++;
-		} else {
-			/* add ESC-OUT code */
-			if (esc_in) {
-			/* printf("3:ESC-OUT[%d]\n", j); */
-				p[j] = 0x1B;
-				p[j+1] = 0x28;
-				p[j+2] = 0x42;
-				j += 3;
-				esc_in = 0;
-			}
-			p[j] = high;
-			/* printf("4:p[%d]=%x\n", j, p[j]); */
-		}
-        }
-	/* add ESC code */
-	if (esc_in) {
-	/* printf("5:ESC-OUT[%d]\n", j); */
-		p[j] = 0x1B;
-		p[j+1] = 0x28;
-		p[j+2] = 0x42;
-		j += 3;
-	}
-        p[j] = '\0';
-
-	return (0);
-}
-
-int euc2jis(char *dst, char *src)
-{
-	int i, j, len, esc_in = 0;    
-	char *p; 
-
-	len = strlen((char *)src);
-	p = dst;
-
-	for (i = 0, j = 0; i < len; i++, j++) {
-		if (src[i] & 0x80) {
-			if (esc_in == 0) {
-				p[j] = 0x1B;
-				p[j+1] = 0x24;
-				p[j+2] = 0x42;
-				j += 3;
-				esc_in = 1;
-			}
-			p[j] = src[i] & 0x7F;
-			p[j+1] = src[i+1] & 0x7F;
-			i++;
-			j++;
-		} else {
-			if (esc_in) {
-				p[j] = 0x1B;
-				p[j+1] = 0x28;
-				p[j+2] = 0x42;
-				j += 3;
-				esc_in = 0;
-			}
-			p[j] = src[i];
-		}
-	}
-	if (esc_in) {
-		p[j] = 0x1B;
-		p[j+1] = 0x28;
-		p[j+2] = 0x42;
-		j += 3;
-	}
-	p[j] = '\0';
-
-	return (0);
-}
-
 int ltgt_encode(char *dst, char *src)
 {
         int i, j, len, esc_in;
@@ -310,32 +284,59 @@ int ltgt_encode(char *dst, char *src)
         return (0);
 }
 
+/* CP932 -> ISO2022-JP */
 int channel_decode(char *dst, char *src)
 {
-	char channel[40];
+	const char *in;
+	char *out;
+	size_t in_len, out_len;
+	char channel[C_NAME];
+	char dst2[C_NAME];
+
+	in = dst2;
+	out = dst;
+	out_len = sizeof(dst2);
 
 	/* %hoge -> #hoge:*.jp */
 	if (src[0] == '%') {
 		sscanf(src, "%%%s", channel);
-		sprintf(dst, "#%s:*.jp", channel);
+		sprintf(dst2, "#%s:*.jp", channel);
 	} else
-		strcpy(dst, src);
+		strcpy(dst2, src);
+
+	/* CP932 -> ISO-2022-JP */
+	in_len = strlen(dst2) + 1;
+	charconv(out, out_len, in, in_len, "iso-2022-jp", "cp932");
 
 	return (0);
 }
 
+/* ISO2022-JP -> CP932 */
 int channel_encode(char *dst, char *src)
 {
 	int n = 0;
-	char channel[40];
+	const char *in;
+	char *out;
+	size_t in_len, out_len;
+	char dst2[C_NAME];
+	char channel[C_NAME];
 	char sufix[3];
+	out_len = sizeof(dst2);
+
+	in = dst2;
+	out = dst;
 
 	/* #hoge:*.jp -> %hoge */
 	if ((n = sscanf(src, "#%[^:]:%*2s%2s",
-			channel, sufix)) == 2)
-		sprintf(dst, "%%%s", channel);
-	else
-		strcpy(dst, src);
+			channel, sufix)) == 2) {
+		sprintf(dst2, "%%%s", channel);
+	} else {
+		strcpy(dst2, src);
+	}
+
+	/* ISO-2022-JP -> CP932 */
+	in_len = strlen(dst2) + 1;
+	charconv(out, out_len, in, in_len, "cp932", "iso-2022-jp");
 
 	return (0);
 }
@@ -347,7 +348,7 @@ const char B64[] =
 char * base64_decode(char *src)
 {
 	int i, j, len;
-	char buf[HTTPBUF];
+	static char buf[IRC_MAX * 2];
 	char *p;
 
 	len = strlen(src);
@@ -430,17 +431,23 @@ int strcmp2(char *str1, char *str2)
 	return tolower(*p) > tolower(*q) ? (1) : (-1);
 }
 
-#if 0
-int count_char(char *str)
+int count_user(char *str)
 {
-	char *p;
-	int count = 0;
+	char *p, *q;
+	int count = 1;
 
-	for (p = str; *p; p++) {
-		if (*p < 0x20)
-			count++;
+	q = (char*)malloc(strlen(str) + 1);
+        if (q == NULL) return -1;
+
+	strcpy(q, str);
+
+	p = strtok(q, " ");
+	while (p != NULL ) {
+		p = strtok(NULL," ");
+		if (p != NULL) count++;
 	}
 
+	free(q);
+
 	return count;
 }
-#endif
