@@ -1,6 +1,6 @@
-/* $Id: httpd.c,v 1.43 2005/12/16 17:55:25 ichiro Exp $ */
+/* $Id: httpd.c,v 1.84 2006/01/08 10:08:32 ichiro Exp $ */
 /*
- * Copyright (c) 2004
+ * Copyright (c) 2004, 2005, 2006
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
  * All rights reserved.
  *
@@ -48,11 +48,18 @@
 #include <sys/time.h>
 #include <sys/param.h>
 
+extern int debug;
+
 #include "kircd.h"
 #include "httpd.h"
 
 void httpd_proc(struct tparam *, int);
-void print_command_page(struct tparam *, int);
+int auth_proc(int, struct tparam *, char *, char *);
+int easy_regist(int, struct tparam *, char *, char *);
+void print_command_page(struct tparam *);
+
+extern int check_timeout(struct tparam *);
+extern int check_agent(struct tparam *, char *);
 
 gpointer
 httpd(gpointer param)
@@ -82,144 +89,150 @@ void httpd_proc(struct tparam *th2, int socket)
 	struct tm *date;
 	
 	char buf[SBUFLEN];
-	char meth_name[64];
+	char meth_name[M_LEN];
 	char channel[C_NAME], channel2[C_NAME];
-	char uri_addr[64];
-	char base64[HTTPBUF];
-	char basic_auth[HTTPBUF];
-	char header[HTTPBUF];
-	char message[HTTPBUF];
-	char post_message[HTTPBUF + 256]; /* include url_encode mergin */
-	char post_message2[HTTPBUF + 256];
-	char web_user[64], web_pass[64];
-	char cookie_user[64], cookie_pass[64], date_str[40];
+	char uri_addr[URI_LEN];
+	char header[IRC_MAX];
+	char message[IRC_MAX];
+	char post_message[IRC_MAX * 2]; /* include url_encode mergin */
 	char *s, *q;
-	char cmd_cname[C_NAME], cmd_cmd[20], cmd_topic[IRC_MAX];
+	char cmd_cmd[CMD_LEN];
+	char cmd_tsel[CMD_LEN];
 	char cmd_buf[IRC_MAX];
+	char subtopic[SUBTOPIC_LEN];
 
 	int n = 0;
 	int r_fd = 0;
 	char http_ver[4];
 
 	int number = 0;
+	int subusernum = 0;
 	int update = 0;
 	int start_number = 0;
 	int post_length = 0;
 	int post_ch_number = 0;
 
-	unsigned char *p;
-
 	/* bzero */
 	bzero(buf, sizeof(buf));
 	bzero(meth_name, sizeof(meth_name));
 	bzero(uri_addr, sizeof(uri_addr));
-	bzero(base64, sizeof(base64));
-	bzero(basic_auth, sizeof(basic_auth));
+	bzero(header, sizeof(header));
 	bzero(message, sizeof(message));
 	bzero(channel, sizeof(channel));
 	bzero(channel2, sizeof(channel2));
 	bzero(post_message, sizeof(post_message));
-	bzero(post_message2, sizeof(post_message2));
 	bzero(th2->postdata, sizeof(th2->postdata));
 
-	bzero(cmd_cname, sizeof(cmd_cname));
 	bzero(cmd_cmd, sizeof(cmd_cmd));
-	bzero(cmd_topic, sizeof(cmd_topic));
 	bzero(cmd_buf, sizeof(cmd_buf));
+	bzero(subtopic, sizeof(subtopic));
 
+	/* get buffer from stream */
 	if (read(socket, buf, SBUFLEN) <= 0 ) {
 		printf("error: reading a request.\n");
 		return;
 	}
 
-	if ((sscanf(buf, "%s %s HTTP/%s\n",
-			meth_name, uri_addr, http_ver)) < 3) {
+	if (sscanf(buf, "%s %s HTTP/%s\n",
+			meth_name, uri_addr, http_ver) < 3) {
 		printf("error: catch strings of request.\n");
 	}
-#ifdef DEBUG
-	printf("HTTP(DEBUG):meth_name= %s\n", meth_name);
-	printf("HTTP(DEBUG):uri_addr= %s\n", uri_addr);
-	printf("HTTP(DEBUG):http_ver= %s\n", http_ver);
-	printf("---------------------------\n");
-	printf("BUF(DEBUG):%s\n", buf);
 
-#endif
-	/* Use cookie ? */
-	if (th2->use_cookie) {
-	   /* Cookie Authentication */
-	   if ((p = strstr(buf, "Cookie:")) != NULL) {
-		strcpy(base64, p + 8);
-#ifdef DEBUG2
-	printf("DEBUG: Cookie get.\n");
-#endif
+	DPRINTF(1, ("meth_name= %s\n", meth_name));
+	DPRINTF(1, ("uri_addr= %s\n", uri_addr));
+	DPRINTF(1, ("http_ver= %s\n", http_ver));
+	DPRINTF(2, ("---------------------------\n"));
+	DPRINTF(2, ("BUF:%s\n", buf));
 
-		if (sscanf(base64, "user=%[^;]; pass=%[^\r^\n]",
-		    cookie_user, cookie_pass) == 2);
-		else if (sscanf(base64, "pass=%[^;]; user=%[^\r^\n]",
-			 cookie_pass, cookie_user) == 2);
-		else
-			printf("DEBUG:error\n");
-	    }
-	} /* Use cookie */
+	/* init header */
+	strcpy(header, http_header0);
 
-	/* Password Authentication */
-	if ((p = strstr(buf, "Authorization: Basic")) != NULL) {
-#ifdef DEBUG2
-	printf("DEBUG: Passwd get.\n");
-#endif
-		strcpy(base64, p + 21);
-		sscanf(base64, "%[^\r^\n]", basic_auth);
-		sscanf(base64_decode(basic_auth),
-		       "%[^:]:%[^\n]", web_user, web_pass);
-	}
-
-	if (th2->use_cookie &&
-	   (strcmp(cookie_user, th2->auth_user) == 0) &&
-	   (strcmp(cookie_pass, th2->auth_pass) == 0)) {
-#ifdef DEBUG2
-	printf("DEBUG: Cookie Authentication done.\n");
-#endif
-		/* copy header */
-		strcpy(header, http_header0);
-	} else if ((strcmp(web_user, th2->auth_user) == 0) &&
-		   (strcmp(web_pass, th2->auth_pass) == 0)) {
-#ifdef DEBUG2
-	printf("DEBUG: Passwd Authentication done.\n");
-#endif
-		/* copy header */
-		strcpy(header, http_header0);
-
-		/* make cookie info */
-		timer = time(NULL) + th2->cookie_expire * 24 * 60 * 60;
-		date = gmtime(&timer);
-		strftime(date_str, 255, "%a, %d-%b-%Y %H:%M:%S GMT;\n", date);
-
-		/* user */
-		sprintf(message, "Set-Cookie: user=%s; expires=", web_user);
-		strcat(message, date_str);
-		strcat(header, message);
-
-		/* pass */
-		sprintf(message, "Set-Cookie: pass=%s; expires=",web_pass);
-		strcat(message, date_str);
-		strcat(header, message);
-	} else {
-		/* cannot auth */
-		send_msg(socket, th2, http_auth_401_header);
+	/* check user-agent (Easy access) */
+	if ((th2->auth_method == 2) && /* Easy Access */
+	    (check_agent(th2, buf) < 0)) {
+		DPRINTF(0, ("error: Detect PC browser\n"));
+		 /* send data */
+		send_msg(socket, th2, header);
 		return;
 	}
 
-	/* check and clear auth info */
-	bzero(web_user, sizeof(web_user));
-	bzero(web_pass, sizeof(web_pass));
-	bzero(cookie_user, sizeof(cookie_user));
-	bzero(cookie_pass, sizeof(cookie_pass));
+	/* check timeout */
+	if ((th2->auth_method == 2) && /* Easy Access */
+	    (check_timeout(th2) < 0) &&
+	    (strcmp(meth_name, "POST") == 0)) {
+		/* zero clear buffer */
+		bzero(th2->store_message, sizeof(th2->store_message));
+
+		strcat(th2->store_message, buf);
+
+		/* Get length of post data */
+		if ((s = strstr(buf, "Content-Length: ")))
+			post_length = atoi(s + 16);
+
+		/* zero clear */
+		bzero(buf, sizeof(buf));
+		/* read more data */
+		read(socket, buf, post_length);
+		/* add post data to end of buffer */
+		strncat(th2->store_message, buf, post_length);
+		strcat(th2->store_message, "\0");
+
+		DPRINTF(2, ("---\nSTORE POST data(%d): %s\n---\n",
+			strlen(th2->store_message),
+			th2->store_message));
+
+		/* show auth page */
+		print_easybutton(th2);
+		send_msg(socket, th2, header);
+		return;
+	}
 
 	/* POST session */
-	if ((strcmp(meth_name, "POST")) == 0) {
-#ifdef DEBUG
-		printf("status: POST Session\n");
-#endif
+	if (strcmp(meth_name, "POST") == 0) {
+post_start:
+		DPRINTF(0, ("status: POST Session\n"));
+
+	    	/* registration page (EasyAccess) */
+		if (strcmp(uri_addr + 1, "regist") == 0) {
+		    /* EasyAccess registration */
+		    DPRINTF(0, ("EasyAccess registration page,%d,%d\n",
+				th2->regist_done, th2->auth_flag));
+		    if (!th2->regist_done || !th2->auth_flag) {
+			if (easy_regist(socket, th2, buf, header) < 0) {
+			    DPRINTF(0, ("EasyAccess registration Failed\n"));
+			}
+		    }
+
+		    /* check store_message */
+		    if (strlen(th2->store_message) > 0) {
+			if (th2->regist_done && th2->auth_flag) {
+			    DPRINTF(0, ("STORE POST data exist\n"));
+			    /* zero clear buffer */
+			    bzero(buf, sizeof(buf));
+
+			    /* re-write store data */
+			    strcpy(buf, th2->store_message);
+
+			    /* get uri_addr */
+			    DPRINTF(1, ("RE-WRITE URI %s->", uri_addr));
+			    sscanf(buf, "%*s %s HTTP/%*s\n", uri_addr);
+			    DPRINTF(1, ("%s\n", uri_addr));
+
+			    /* zero clear store buffer */
+			    bzero(th2->store_message, sizeof(th2->store_message));
+
+			    goto post_start;
+			} else {
+			    DPRINTF(0, ("Delete STORE POST data\n"));
+			    /* zero clear store buffer */
+			    bzero(th2->store_message, sizeof(th2->store_message));
+			}
+		    }
+		    DPRINTF(0, ("goto index_start\n"));
+		    strcpy(uri_addr, "/");
+		    goto index_start;
+		}
+
 		/* wait for flags clear */
 		while ((!th2->t_empty) || (th2->t_lock)) {
 			usleep(10000); /* 10ms */
@@ -235,9 +248,8 @@ void httpd_proc(struct tparam *th2, int socket)
 			/* zero clear buffer */
 			bzero(buf, sizeof(buf));
 			read(socket, buf, post_length);
-#ifdef DEBUG
-			printf("%s\n", buf);
-#endif
+
+			DPRINTF(0, ("POST data: %s\n", buf));
 		}
 
 		/* Get post message */
@@ -247,16 +259,14 @@ void httpd_proc(struct tparam *th2, int socket)
 
 			if (post_length < 3) {
 				/* clear trans. empty flag */
-				th2->t_empty = 0;
+				th2->t_empty = 1;
                                 th2->t_lock = 0;
                                 goto index_start;
 			}
 
 			/* URL decord */
-			url_decode_jis(post_message2, post_message);
-#ifdef DEBUG
-			printf("decode: %s\n", post_message);
-#endif
+			url_decode_jis(post_message);
+			DPRINTF(2, ("post msg decode: %s\n", post_message));
 
 			/* Get channel number to post */
 			sscanf(uri_addr + 1, "channel%d",
@@ -270,7 +280,7 @@ void httpd_proc(struct tparam *th2, int socket)
 			sprintf(th2->postdata,
 				":%s PRIVMSG %s :%s\r\n",
 				 th2->real_nickname, channel2,
-				 post_message2);
+				 post_message);
 
 			/* store own messages */
 			strcpy(th2->l_buff, th2->postdata);
@@ -285,15 +295,16 @@ void httpd_proc(struct tparam *th2, int socket)
 			th2->t_empty = 0;
 			th2->t_lock = 0;
 
+			goto index_start;
 		} else if ((s = strstr(buf, "cname="))) {
 			/* clear buffer */
-			bzero(cmd_cname, sizeof(cmd_cname));        
 			bzero(cmd_cmd, sizeof(cmd_cmd));    
-			bzero(cmd_topic, sizeof(cmd_topic));
+			bzero(cmd_tsel, sizeof(cmd_tsel));
 
-			if (post_length < 22) {
-				/* clear trans. empty flag */
-				th2->t_empty = 0;
+			if (post_length < 39) {
+			   /* 39: cname=&cname2=&cmd=join&topic=&t_sel=on */
+			   /* clear trans. empty flag */
+				th2->t_empty = 1;
                                 th2->t_lock = 0;
                                 goto index_start;
 			}
@@ -301,15 +312,14 @@ void httpd_proc(struct tparam *th2, int socket)
 			sprintf(post_message,	/* XXX temporary used */
 				"%.*s", post_length, s);
 
-			/* cname=&cname2=&cmd=part&topic= */
 			q = strtok(post_message, "&");
-			if (q != NULL) strcpy(cmd_cname, q + 6);
+			if (q != NULL) strcpy(channel, q + 6);
 							/* 6: cname= */
 
 			q = strtok(NULL, "&");
 			if (q != NULL) {
-				if (strlen(cmd_cname) == 0)
-					strcpy(cmd_cname, q + 7);
+				if (strlen(channel) == 0)
+					strcpy(channel, q + 7);
 							/* 7: cname2= */
 			}
 
@@ -322,20 +332,25 @@ void httpd_proc(struct tparam *th2, int socket)
 			if (q != NULL) {
 				strcpy(cmd_buf, q + 6); /* 6: topic= */
 			}
-#ifdef DEBUG
-			printf("cname=%s\ncmd=%s\ntopic=%s\n",
-				cmd_cname, cmd_cmd, cmd_buf);
-#endif
+
+			q = strtok(NULL, "&");
+			if (q != NULL) {
+				strcpy(cmd_tsel, q + 6);
+			}				/* 6: t_sel= */
+
+			DPRINTF(0, ("cname=%s\ncmd=%s\ntopic=%s\nt_sel=%s\n",
+				channel, cmd_cmd, cmd_buf, cmd_tsel));
+
 			/* make command message */
 			if (strcmp(cmd_cmd, "join") == 0) {
-			   if(strlen(cmd_cname) > 0){
-				url_decode_rawdata(channel, cmd_cname);
+			   if(strlen(channel) > 0){
+				url_decode_jis(channel);
 				/* send join */
 				send_join(th2, channel);
 			   }
 			} else if (strcmp(cmd_cmd, "part") == 0) {
-			   if(strlen(cmd_cname) > 0){
-				url_decode_rawdata(channel, cmd_cname);
+			   if(strlen(channel) > 0){
+				url_decode_jis(channel);
 				/* send part */
 				send_part(th2, channel);
 			   }
@@ -345,32 +360,38 @@ void httpd_proc(struct tparam *th2, int socket)
 					th2->real_nickname, cmd_buf);
 			   }
 			} else if (strcmp(cmd_cmd, "topic") == 0) {
-			   if((strlen(cmd_cname) > 0) &&
+			   if((strlen(channel) > 0) &&
 				(strlen(cmd_buf) > 0)){
-				url_decode_rawdata(channel, cmd_cname);
 				/* any -> #hoge*.jp */
+				url_decode_jis(channel);
 				channel_decode(channel2, channel);
-				url_decode_jis(cmd_topic, cmd_buf);
+				/* decode */
+				url_decode_jis(cmd_buf);
 				sprintf(th2->postdata, ":%s TOPIC %s :%s\r\n",
 					th2->real_nickname,
-					channel2, cmd_topic);
+					channel2, cmd_buf);
 			   }
-			} else {
-				goto command_start;
 			}
-		}
 
-		/* clear trans lock and empty flag */
-		th2->t_empty = 0;
-		th2->t_lock = 0;
+			if (strcmp(cmd_tsel, "on") == 0) {
+				/* change to show topic */
+				th2->show_topic = 1;
+			} else if (strcmp(cmd_tsel, "off") == 0) {
+				th2->show_topic = 0;
+			}
+
+			/* clear trans lock and empty flag */
+			th2->t_empty = 0;
+			th2->t_lock = 0;
+
+			goto command_start;
+		}
 	}	/* POST */
 
 #if 0
 	/* favicon.ico */
 	if (strstr(uri_addr + 1, "favicon.ico")) {
-#ifdef DEBUG
-		printf("favicon session\n");
-#endif
+		DPRINTF(0, ("favicon session\n"));
 		if ((r_fd = open(uri_addr, O_RDONLY,
 				 0666)) == -1) {
 			printf("favicon.ico file not found\n");
@@ -391,28 +412,101 @@ void httpd_proc(struct tparam *th2, int socket)
 		send_msg(socket, http_footer);
 	} /* favicon.ico */
 #endif
+	/* GET */
+	if (strcmp(meth_name, "GET") == 0) {
+index_start:
+	    /* main entrance (authentication) */
+	    if (auth_proc(socket, th2, buf, header) < 0) {
+		if (strlen(th2->store_message) > 0) {
+			DPRINTF(1, ("Clear store_message\n"));
+			bzero(th2->store_message, sizeof(th2->store_message));
+		}
+		return;
+	    }
 
-	/* start COMMAND page */
+	    /* start UserList Sub page (GET) */
+	    if (strcmp(uri_addr + 1, "user") == 0) {
+		DPRINTF(0, ("UserList Sub page\n"));
+
+		sprintf(message, "User list / Topic<br>");
+		send_buf(th2, message);
+
+		for (n = 0; n <= th2->ch_count; n++) {
+		   if (strlen(th2->topic[n]) > 0) {
+			strncpy(subtopic, th2->topic[n], 14);
+			if (strlen(th2->topic[n]) > 14) {
+				strcat(subtopic, "...\0");
+			} else {
+				strcat(subtopic, "\0");
+			}
+		   }
+			if (n < 9) {
+			    sprintf(message,
+			    "%d <a accesskey=\"%d\" href=\"subuser%d\">%s</a> "
+			    "[%d] <a href=\"subtopic%d\">%s</a><br>",
+			    n, n, n, th2->name[n], th2->member_count[n],
+			    n, subtopic);
+			} else {
+			    sprintf(message,
+			    "* <a href=\"subuser%d\">%s</a> "
+			    "[%d] <a href=\"subtopic%d\">%s</a><br>",
+			    n, th2->name[n], th2->member_count[n],
+			    n, subtopic);
+			}           
+			send_buf(th2, message);
+
+			strcpy(subtopic, "\0");
+		}
+
+		send_buf(th2,
+			"<hr><a accesskey=\"9\" href=\"/\">back[9]</a>");
+		send_msg(socket, th2, header);
+		return;
+	    }
+
+	    /* start UserList SubSub page (GET) */
+            if ((sscanf(uri_addr + 1, "subuser%d", &subusernum)) == 1) {
+		DPRINTF(0, ("UserList SubSub page\n"));
+
+		sprintf(message,
+		        "users:%s<br>", th2->member_names[subusernum]);
+		send_buf(th2, message);
+
+		send_buf(th2,
+			"<hr><a accesskey=\"9\" href=\"/\">back[9]</a>");
+		send_msg(socket, th2, header);
+
+		return;
+	    }
+
+	    /* start topic jump from subtopic tag (GET) */
+	    if (sscanf(uri_addr + 1, "subtopic%d", &subusernum) == 1) {
+		DPRINTF(0, ("SubTopic page\n"));
+
+		sprintf(message,
+			"Topic:%s<br>", th2->topic[subusernum]);
+		send_buf(th2, message);
+
+		send_msg(socket, th2, header);
+
+		return;
+	    }
+
+	    /* start COMMAND page */
+	    if (strcmp(uri_addr + 1, "cmd") == 0) {
 command_start:
-	if ((strcmp(uri_addr + 1, "cmd") == 0) &&
-	    (strcmp(meth_name, "GET") == 0)) {
-#ifdef DEBUG
-		printf("COMMAND page\n");
-#endif
-		print_command_page(th2, socket);
+		DPRINTF(0, ("COMMAND page\n"));
+		print_command_page(th2);
 
 		/* send data */
 		send_msg(socket, th2, header);
 
 		return;
-	}
+	    }
 
-index_start:
-	/* start index of each channels */
-	if ((sscanf(uri_addr + 1, "channel%d", &number)) == 1) {
-#ifdef DEBUG
-		printf("Sub index session\n");
-#endif
+	    /* start index of each channels */
+	    if (sscanf(uri_addr + 1, "channel%d", &number) == 1) {
+		DPRINTF(0, ("Sub index session\n"));
 		/* check update or not */
 		if (number >= 50) {
 			update = 1;
@@ -420,17 +514,11 @@ index_start:
 		}
 
 		/* print TOPIC */
-		if (strlen(th2->topic[number]) > 0) {
-			sprintf(message, "$B$H$T$/(B :%s<br>",
-				th2->topic[number]);
-			send_buf(th2, message);
-		}
-
-		/* print NAMES */
-		if (strlen(th2->member_names[number]) > 0) {
-			sprintf(message, "users:%s<br>",
-				th2->member_names[number]);
+		if (th2->show_topic) {
+		    if (strlen(th2->topic[number]) > 0) {
+			sprintf(message, "%s<br>", th2->topic[number]);
 			send_buf(th2, message);
+		    }
 		}
 
 		/* print FORM */
@@ -438,9 +526,9 @@ index_start:
 			"<form action=\"/channel%d\" method=\"post\">", number);
 		send_buf(th2, message);
 		send_buf(th2,
-			 "<input type=text name=\"m\" size=\"10\">");
+			 "<input type=\"text\" name=\"m\" size=\"10\">");
 		send_buf(th2,
-			 "<input type=submit value=\" $BAw?.(B \"></form>");
+			 "<input type=\"submit\" value=\"send\"></form>");
 		send_buf(th2,
 			"<a accesskey=\"8\" href=\"/\">back[8]</a><hr>");
 
@@ -471,41 +559,72 @@ index_start:
 		update = 0;
 
 		return;
-	}
-
-	/* start main index */
-#ifdef DEBUG
-	printf("start main index\n");
-#endif
-	for (n = 0; n <= th2->ch_count; n++) {
-	    if (n < 9) {
-	       sprintf(message,
-		   "%d <a accesskey=\"%d\" href=\"channel%d\">%s</a>"
-		   "<a href=\"channel%d\">(%d)</a><br>",
-		    n, n, n, th2->name[n],
-		    n + 50, th2->history_count[n] -
-		    th2->read_count[n]);
-	    } else {
-	sprintf(message,
-		   "* <a href=\"channel%d\">%s</a>"
-		   "<a href=\"channel%d\">(%d)</a><br>",
-		    n, th2->name[n],
-		    n + 50, th2->history_count[n] -
-		    th2->read_count[n]); 	
 	    }
+
+	    /* start main index */
+	    if (strcmp(uri_addr, "/") == 0) {
+	    DPRINTF(0, ("start main index\n"));
+		for (n = 0; n <= th2->ch_count; n++) {
+		    if (n < 9) {
+			sprintf(message,
+			    "%d <a accesskey=\"%d\" href=\"channel%d\">%s</a> "
+			    "<a href=\"channel%d\">(%d)</a><br>",
+			    n, n, n, th2->name[n],
+			    n + 50, th2->history_count[n] -
+			    th2->read_count[n]);
+		    } else {
+			sprintf(message,
+			    "* <a href=\"channel%d\">%s</a> "
+			    "<a href=\"channel%d\">(%d)</a><br>",
+			    n, th2->name[n],
+			    n + 50, th2->history_count[n] -
+			    th2->read_count[n]); 	
+		    }
+		    send_buf(th2, message);
+		}
+		sprintf(message,
+		    "<hr>9 <a accesskey=9 href=\"cmd\">"
+		    "cmd page</a><br>");
 		send_buf(th2, message);
-	}
-	sprintf(message,
-		"<hr>9 <a accesskey=9 href=\"cmd\">"
-		"command page[9]</a><br>");
-	send_buf(th2, message);
-	sprintf(message,
-		"<br><left>kircd Ver %s </left>", VERSION);
-	send_buf(th2, message);
 
-	send_msg(socket, th2, header);
+		sprintf(message,
+		    "* <a href=\"user\">user/topic</a><br>");
+		send_buf(th2, message);
 
-	return;
+		sprintf(message,
+		    "<br><left>kircd Ver %s </left>", VERSION);
+		send_buf(th2, message);
+
+		send_msg(socket, th2, header);
+
+		return;
+	    }
+
+	    /* logout */
+	    if (th2->auth_method == 2) { /* Easy Access */
+		if (strcmp(uri_addr + 1, "logout") == 0) {
+		    DPRINTF(0, ("status: LOGOUT Session\n"));
+		    th2->auth_flag = 0;
+
+		    send_buf(th2, "logout done...");
+		    send_msg(socket, th2, header);
+
+		    return;
+		}
+		if (strcmp(uri_addr + 1, "logoff") == 0) {
+		    DPRINTF(0, ("status: LOGOFF Session\n"));
+		    th2->auth_flag = 0;
+		    th2->regist_done = 0;
+		    bzero(th2->uid, sizeof(th2->uid));
+
+		    send_buf(th2, "logoff done...");
+		    send_buf(th2, "Please register again.");
+		    send_msg(socket, th2, header);
+
+		    return;
+		}
+	    }
+	} /* GET */
 }
 
 void send_buf(struct tparam *th, char *addmsg)
@@ -516,15 +635,11 @@ void send_buf(struct tparam *th, char *addmsg)
 	add_len = strlen(addmsg);
 
 	if (th->send_buflen < (buf_len + add_len + 1)) {
-#ifdef DEBUG
-		printf("SEND_BUF %d -> ", th->send_buf); 
-#endif
+		DPRINTF(1, ("SEND_BUF %d -> ", th->send_buf));
 		th->send_buflen = th->send_buflen + HTTPBUF;
 		th->send_buf = (char *)realloc(th->send_buf,
 				th->send_buflen);
-#ifdef DEBUG
-		printf("%d\n", th->send_buf); 
-#endif
+		DPRINTF(1, ("%d\n", th->send_buf));
 	}
 
 	strcat(th->send_buf, addmsg);
@@ -552,10 +667,9 @@ int send_msg(int fd, struct tparam *th, char *header_msg)
 	buffer = (char*)malloc(total_len + 1);
 	*buffer = '\0';
 
-#ifdef DEBUG
-	printf("send_msg: length = %d\r\n\r\n",
-		header_len + msg_len + footer_len);
-#endif
+	DPRINTF(0, ("send_msg: length = %d\r\n\r\n",
+		header_len + msg_len + footer_len));
+
 	/* copy control header */
 	strcat(buffer, header_msg);
 	strcat(buffer, http_contype);
@@ -572,9 +686,7 @@ int send_msg(int fd, struct tparam *th, char *header_msg)
 
 	buffer_len = strlen(buffer);
 
-#ifdef DEBUG
-	printf("SEND_DATA:(%d)byte\n%s\n", buffer_len, buffer);
-#endif
+	DPRINTF(2, ("SEND_DATA:(%d)byte\n%s\n", buffer_len, buffer));
 
 	if (write(fd, buffer, buffer_len) != buffer_len){
 		printf("error: writing %d chars\n", buffer_len);
@@ -589,7 +701,7 @@ int send_msg(int fd, struct tparam *th, char *header_msg)
 	return buffer_len;
 }
 
-void print_command_page(struct tparam *th2, int socket)
+void print_command_page(struct tparam *th2)
 {
 	char str[C_NAME];
 	int i;
@@ -598,10 +710,8 @@ void print_command_page(struct tparam *th2, int socket)
 	   "<form action=\"/cmd\" method=\"post\">");
 	
 	send_buf(th2, "Channel:");
-	send_buf(th2,
-	   "<br><select name=\"cname\">");
-	send_buf(th2,
-	   "<option value=\"\">specific</option>");
+	send_buf(th2, "<br><select name=\"cname\">");
+	send_buf(th2, "<option value=\"\">specific</option>");
 	for (i = 0; i < th2->ch_count + 1; i++) {
 		sprintf(str, "<option value=\"%s\">%s</option>",
 			th2->name[i], th2->name[i]);
@@ -610,24 +720,39 @@ void print_command_page(struct tparam *th2, int socket)
 	send_buf(th2, "</select>");
 
 	send_buf(th2,
-	   "<br><input type=text name=\"cname2\" size=\"17\">");
+	   "<br>"
+	   "<input type=\"text\" name=\"cname2\" size=\"17\">"
+	   "<br>"
+	   "<input type=\"radio\" name=\"cmd\" value=\"part\">PART"
+	   "<input type=\"radio\" name=\"cmd\" value=\"join\" CHECKED>JOIN"
+	   "<hr>"
+	   "<input type=\"radio\" name=\"cmd\" value=\"topic\">TOPIC"
+	   "<input type=\"radio\" name=\"cmd\" value=\"nick\">NICK"
+	   "<br>"
+	   "<input type=\"text\" name=\"topic\" size=\"17\">");
+
+	send_buf(th2, "<hr>Show Topic");
+
+	if (th2->show_topic) {
+	   send_buf(th2,
+	      "<input type=\"radio\" name=\"t_sel\" value=\"on\" CHECKED>On"
+	      "<input type=\"radio\" name=\"t_sel\" value=\"off\">Off");
+	} else {
+	   send_buf(th2,
+	      "<input type=\"radio\" name=\"t_sel\" value=\"on\">On"
+	      "<input type=\"radio\" name=\"t_sel\" value=\"off\" CHECKED>Off");
+	}
 
 	send_buf(th2,
-	   "<br><input type=radio name=\"cmd\" value=\"part\">PART");
-	send_buf(th2,
-	   "<input type=radio name=\"cmd\" value=\"join\">JOIN");
-
-	send_buf(th2,
-	   "<hr><input type=radio name=\"cmd\" value=\"topic\">TOPIC");
-	send_buf(th2,
-	   "<input type=radio name=\"cmd\" value=\"nick\">NICK");
-	send_buf(th2,
-	   "<br><input type=text name=\"topic\" size=\"17\">");
-
-	send_buf(th2,
-	   "<hr><input type=submit value=\" $BAw?.(B [1]\">");
-	send_buf(th2,
+	   "<hr><input type=\"submit\" value=\"send\">"
 	   "<a accesskey=\"8\" href=\"/\"> back[8]</a></form>");
+
+	if (th2->auth_method == 2) { /* Easy Access */
+		send_buf(th2,
+			"<hr><a href=\"logout\">logout</a><br>");
+		send_buf(th2,
+			"<hr><a href=\"logoff\">logoff</a><br>");
+	}
 }
 
 int send_join(struct tparam *th, char *channel)
@@ -635,23 +760,22 @@ int send_join(struct tparam *th, char *channel)
 	int i;
 	char channel2[C_NAME];
 
-	/* any ->%hoge */
+	DPRINTF(0, ("send_join()\n"));
+
+	/* any ->%hoge(CP932) */
 	channel_encode(channel2, channel);
 
 	/* search exist channel */
 	for (i = 0; i <= th->ch_count; i++) {
 		if (strcmp2(th->name[i], channel2) == 0) {
-#ifdef DEBUG
-			printf("channel exist!\n");
-#endif
+			DPRINTF(0, ("channel exist!\n"));
 			return (0);
 		}
 	}
 
-	/* %hoge -> #hoge*.jp */
-	channel_decode(channel2, channel);
-
-	if (channel2[0] == '#') { /* make channel */
+	if (channel2[0] == '%') { /* make channel */
+		/* %hoge -> #hoge*.jp */
+		channel_decode(channel2, channel);
 		sprintf(th->postdata, ":%s JOIN %s\r\n",
 			th->real_nickname, channel2);
 	} else {			/* make channel with person */
@@ -667,13 +791,16 @@ int send_part(struct tparam *th, char *channel)
 {
 	char channel2[C_NAME];
 
-	/* any -> #hoge*.jp */
-	channel_decode(channel2, channel);
+	DPRINTF(0, ("send_part()\n"));
 
-	if (channel2[0] == '#') {	/* disconnect from irc-channel */
+	if (channel[0] == '%') {	/* disconnect from irc-channel */
+		/* any -> #hoge*.jp */
+		channel_decode(channel2, channel);
 		sprintf(th->postdata, ":%s PART %s\r\n",
 			th->real_nickname, channel2);
 	} else {			/* disconnect from person */
+		/* any ->%hoge(CP932) */
+		channel_encode(channel2, channel);
 		/* delete channel */
 		channel_delete(th, channel2);
 	}
