@@ -1,6 +1,6 @@
-/* $Id: keitai.c,v 1.41 2005/12/16 17:55:25 ichiro Exp $ */
+/* $Id: keitai.c,v 1.62 2006/01/08 09:21:47 ichiro Exp $ */
 /*
- * Copyright (c) 2004
+ * Copyright (c) 2004, 2005, 2006
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
  * All rights reserved.
  *
@@ -45,14 +45,20 @@
 #include <net/if.h>
 #include <sys/param.h>
 
+extern int debug;
+
 #include "kircd.h"
 
+#ifdef HAVE_ICONV
+#include <iconv.h>
+#endif
+
 int
 keitai_start(struct tparam *th)
 {
-#ifdef DEBUG
-		printf("%s\r\n", th->l_buff);
-#endif
+	/* debug */
+	DPRINTF(2, ("RECV_DATA:(%s)\r\n", th->l_buff));
+
 	/* skip NOTICE messages */
 	if (strstr(th->l_buff, "NOTICE")) {
 	}
@@ -78,9 +84,7 @@ keitai_start(struct tparam *th)
 
 	/* catch QUIT */
 	if (strstr(th->l_buff, "QUIT")) {
-#ifdef DEBUG
-		printf("QUIT:%s\n", th->l_buff);
-#endif
+		DPRINTF(0, ("QUIT:%s\n", th->l_buff));
 	}
 
 	/* detect TOPIC */
@@ -89,9 +93,7 @@ keitai_start(struct tparam *th)
 
 	/* MODE */
 	if (strstr(th->l_buff, "MODE")) {
-#ifdef DEBUG
-		printf("MODE:%s\n", th->l_buff);
-#endif
+		DPRINTF(0, ("MODE:%s\n", th->l_buff));
 	}
 
 	/* NAMES */
@@ -114,6 +116,11 @@ int priv_recv(struct tparam *th)
 	int ch_number = 255; /* 255 means "not found" */
 	tm = localtime(&t);
 
+	/* iconv */
+	const char *p_in;
+	char *p_out;
+	size_t in_len, out_len;
+
 	/* zero clear */
 	bzero(channel,  sizeof(channel));
 	bzero(channel2, sizeof(channel2));
@@ -121,26 +128,36 @@ int priv_recv(struct tparam *th)
 	bzero(message2, sizeof(message2));
 
 	/* get PRIVMSG */
-	n = sscanf(th->l_buff, ":%[^!]!%*s PRIVMSG %s :%[^\n]",
+	n = sscanf(th->l_buff, ":%[^!]!%*s PRIVMSG %s :%[^\r\n]",
 			id, channel, message);
-	if (n < 3) { /* for madoka4.2 */
-		n = sscanf(th->l_buff, ":%s PRIVMSG %s :%[^\n]",
+	if ((n == 2) && (strcmp(message, "\0") == 0)) { /* type 0 */
+		strcpy(message, "0");
+	} else if ((n == 1) && (strcmp(message, "\0") == 0)) {
+		/* for madoka4.2 */
+		n = sscanf(th->l_buff, ":%s PRIVMSG %s :%[^\r\n]",
 			id, channel, message);
 	}
 
-	/* channel;  #hoge:*.jp -> %hoge */
-	channel_encode(channel2, channel);
+	/* CTCP */
+	if ((strcmp2(th->real_nickname, channel) == 0) &&
+	    (message[0] == 0x01) &&
+	    (message[strlen(message)-1] == 0x01)) {
+		ctcp_recv(th, id, message);
+		return (0);
+	}
 
 	/* message; '<','>' ---> '&lt;','&gt;' */
         ltgt_encode(message2, message);
 
+	/* conv charset JIS to CP932 */
+	decode_jis2cp932(message2);
+
+	/* channel;  #hoge:*.jp -> %hoge(CP932) */
+	channel_encode(channel2, channel);
+
 	/* add href tag */
 	href_conv(message2);
 
-#ifdef DEBUG 
-	printf("%s | <%s> %s\n", channel2, id, message2);
-#endif
-
 	for (n = 0; n <= th->ch_count; n++) {
 		if (strcmp2(th->name[n], channel2) == 0) {
 					/* message to channel */
@@ -155,15 +172,11 @@ int priv_recv(struct tparam *th)
 	/* can not found out channel, maybe conversation
 	   with another person ? */
 	if (ch_number == 255) {
-		if (strcmp2(th->real_nickname, channel2) == 0) {
+		if (strcmp2(th->real_nickname, channel) == 0) {
 			ch_number = channel_make(th, id);
-#ifdef DEBUG
-			printf("make new channel with %s\n", id);
-#endif
+			DPRINTF(0, ("make new channel with %s\n", id));
 		} else {
-#ifdef DEBUG
-			printf("channel not found(%s)\n", channel2);
-#endif
+			DPRINTF(0, ("channel not found(%s)\n", channel2));
 			return (1);
 		}
 	} 
@@ -196,39 +209,76 @@ int priv_recv(struct tparam *th)
 	}
 	th->history_count[ch_number]++;
 
-#ifdef DEBUG
-	printf("%d: %s | %s\n", (th->history_count[ch_number] - 1), channel2,
-		th->data[ch_number][th->history_count[ch_number] - 1]);
-#endif
+	/* debug */
+	DPRINTF(1, ("(%2d): <CH:%s> | %s\n",
+		(th->history_count[ch_number] - 1), channel2,
+		th->data[ch_number][th->history_count[ch_number] - 1]));
 
 	return (0);
 }
 
+int ctcp_recv(struct tparam *th, char *id, char *message)
+{
+	char *ctcp_mes, data[IRC_MAX];
+	int len;
+
+	len = strlen(message);
+
+	ctcp_mes = (char*)malloc(len - 2 + 1);
+        if (ctcp_mes == NULL) return (0);
+        *ctcp_mes = '\0';
+        strncpy(ctcp_mes, &message[1], len - 2);
+
+	DPRINTF(0, ("CTCP message:%s\n", ctcp_mes));
+
+	if (strcmp2(ctcp_mes, "VERSION") == 0) {
+		/* 0x01VERSION #1:#2:#3 0x01
+		   #1: client name 
+		   #2: client version
+		   #3: env of client */
+		sprintf(data,
+		    ":%s NOTICE %s :%cVERSION %s %s %c\r\n",
+		    th->real_nickname, id, 0x01, PACKAGE, VERSION, 0x01);
+	}
+
+	/* transfer message */
+	send_postdata(th->socket_number, data);
+	DPRINTF(0, ("CTCP send: %s\n", data));
+
+	free(ctcp_mes);
+	return (0);
+}
+
 int names_recv(struct tparam *th)
 {
 	int n = 0;
+	int count = 0;
 	char id[IRC_MAX];
 	char channel[C_NAME], channel2[C_NAME];
 	char names[IRC_MAX];
 
 	/* get NAMES */
-	if ((sscanf(th->l_buff, ":%*s 353 %s %*s %s :%[^\n]",
+	if ((sscanf(th->l_buff, ":%*s 353 %s %*s %s :%[^\r\n]",
 		id, channel, names)) != 3)
 		return (1);
 
 	if (strcmp2(th->real_nickname, id) != 0)
 		return (0);
 
-	/* channel;  #hoge:*.jp -> %hoge */
+	/* channel;  #hoge:*.jp -> %hoge(CP932) */
 	channel_encode(channel2, channel);
 
-#ifdef DEBUG
-	printf("NAMES:channel=%s, names=%s\n", channel2, names);
-#endif
+	/* count user number */
+	count = count_user(names);
+
+	DPRINTF(0, ("NAMES:channel=%s, names(%d)=%s\n",
+			channel2, count, names));
 
 	for (n = 0; n <= th->ch_count; n++) {
-		if (strcmp2(th->name[n], channel2) == 0)
+		if (strcmp2(th->name[n], channel2) == 0) {
                         sprintf(th->member_names[n], " %s", names);
+			th->member_count[n] = count;
+		}
         }
 
 	return (0);
@@ -240,6 +290,11 @@ int topic_recv(struct tparam *th)
 	char topic[IRC_MAX * 2], buf[IRC_MAX * 2];
 	char channel[C_NAME], channel2[C_NAME];
 
+	/* iconv */
+	const char *p_in;
+	char *p_out;
+	size_t in_len, out_len;
+
 	/* zero clear */
 	bzero(buf, sizeof(buf));
 	bzero(channel, sizeof(channel));
@@ -247,24 +302,26 @@ int topic_recv(struct tparam *th)
 
 	/* get TOPIC */
 	if (strstr(th->l_buff, "332")) {
-		if ((sscanf(th->l_buff, ":%*s 332 %*s %s :%[^\n]",
+		if ((sscanf(th->l_buff, ":%*s 332 %*s %s :%[^\r\n]",
 			channel, topic)) != 2)
 			return (1);
 	} else if (strstr(th->l_buff, "TOPIC")) {
-		if ((sscanf(th->l_buff, ":%*s TOPIC %s :%[^\n]",
+		if ((sscanf(th->l_buff, ":%*s TOPIC %s :%[^\r\n]",
 			channel, topic)) != 2)
 			return (1);
 	}
 
-	/* channel;  #hoge:*.jp -> %hoge */
-	channel_encode(channel2, channel);
-
-#ifdef DEBUG
-	printf("TOPIC:channel=%s, topic=%s\n", channel2, topic);
-#endif
 	/* '<','>' ---> '&lt;','&gt;' */
 	ltgt_encode(buf, topic);
 
+	/* conv charset JIS to CP932 */
+	decode_jis2cp932(buf);
+
+	/* channel;  #hoge:*.jp -> %hoge(CP932) */
+	channel_encode(channel2, channel);
+
+	DPRINTF(0, ("TOPIC:channel=%s, topic=%s\n", channel2, topic));
+
 	/* add href tag */
 	href_conv(buf);
 
@@ -299,7 +356,7 @@ int part_recv(struct tparam *th)
 		}
 	}
 
-	/* channel;  #hoge:*.jp -> %hoge */
+	/* channel;  #hoge:*.jp -> %hoge(CP932) */
 	channel_encode(channel2, channel);
 
 	/* delete channel */
@@ -312,6 +369,8 @@ int channel_delete(struct tparam *th, char *channel)
 {
 	int i, j, match = 255;
 
+	DPRINTF(0, ("channel_delete()\n"));
+
 	 /* search exist channel */
 	for (i = 0; i <= th->ch_count; i++) {
 		if (strcmp2(th->name[i], channel) == 0) {
@@ -327,6 +386,7 @@ int channel_delete(struct tparam *th, char *channel)
 		bzero(th->name[match], sizeof(th->name[match]));
 		bzero(th->topic[match], sizeof(th->topic[match]));
 		bzero(th->member_names[match], sizeof(th->member_names[match]));
+		th->member_count[match] = 0;
 		bzero(th->data[match], sizeof(th->data[match]));
 		th->history_count[match] = 0;
 		th->read_count[match] = 0;
@@ -335,11 +395,13 @@ int channel_delete(struct tparam *th, char *channel)
 		bzero(th->name[i], sizeof(th->name[i]));
 		bzero(th->topic[i], sizeof(th->topic[i]));
 		bzero(th->member_names[i], sizeof(th->member_names[i]));
+		th->member_count[i] = 0;
 		bzero(th->data[i], sizeof(th->data[i]));
 
 		strcpy(th->name[i], th->name[i + 1]);
 		strcpy(th->topic[i], th->topic[i + 1]);
 		strcpy(th->member_names[i], th->member_names[i + 1]);
+		th->member_count[i] = th->member_count[i + 1];
 
 		for (j = 0; j < th->history_count[i + 1]; j++) {
 			strcpy(th->data[i][j], th->data[i + 1][j]);
@@ -353,6 +415,7 @@ int channel_delete(struct tparam *th, char *channel)
 		   sizeof(th->topic[th->ch_count]));
 	    bzero(th->member_names[th->ch_count],
 		   sizeof(th->member_names[th->ch_count]));
+	    th->member_count[th->ch_count] = 0;
 	    bzero(th->data[th->ch_count],
 		   sizeof(th->data[match]));
 	    th->history_count[th->ch_count] = 0;
@@ -360,9 +423,7 @@ int channel_delete(struct tparam *th, char *channel)
 	}
 	th->ch_count--;
 
-#ifdef DEBUG
-	printf("part channel name(%d) = %s\n", th->ch_count, channel);
-#endif
+	DPRINTF(0, ("part channel name(%d) = %s\n", th->ch_count, channel));
 
 	return (0);
 }
@@ -387,14 +448,12 @@ int join_recv(struct tparam *th)
 				id, channel) == 2);
 
 		if (strcmp2(th->real_nickname, id) == 0) {
-			/* channel;  #hoge:*.jp -> %hoge */
+			/* channel;  #hoge:*.jp -> %hoge(CP932) */
 			channel_encode(channel2, channel);
 			/* search exist channel */
 			for (i = 0; i <= th->ch_count; i++) {
 				if (strcmp2(th->name[i], channel) == 0) {
-#ifdef DEBUG
-					printf("channel exist!\n");
-#endif
+					DPRINTF(0, ("channel exist!\n"));
 					return (0);
 				}
 			}
@@ -421,9 +480,8 @@ int channel_make(struct tparam *th, char *channel)
 
 	strcpy(th->name[n], channel);
 
-#ifdef DEBUG
-	printf("join channel name(%d) = %s\n", n, th->name[n]);
-#endif
+	DPRINTF(0, ("channel_make(%d) = %s\n", n, th->name[n]));
+
 	return (n);
 }
 
@@ -435,13 +493,13 @@ int ping_recv (struct tparam *th)
 	bzero(server,  sizeof(server));
 
 	/* get PING */
-	sscanf(th->l_buff, "PING :%[^\n]", server);
+	sscanf(th->l_buff, "PING :%[^\r\n]", server);
 
 	/* transfer message */
 	sprintf(data, "PONG %s\r\n", server);
-#ifdef DEBUG
-	printf("PING:sent data\n");
-#endif
+
+	DPRINTF(0, ("PING:sent data\n"));
+
 	send_postdata(th->socket_number, data);
 
 	return (0);
@@ -457,9 +515,9 @@ int nick_recv(struct tparam *th)
 
 	if (strcmp2(th->real_nickname, old_nickname) == 0) {
 		strcpy(th->real_nickname, new_nickname);
-#ifdef DEBUG
-	printf("NICK:%s -> %s\n", old_nickname, th->real_nickname);
-#endif
+	DPRINTF(0, ("NICK:%s -> %s\n",
+		old_nickname, th->real_nickname));
+
 	}
 
 	return (0);
@@ -471,9 +529,8 @@ int send_names(struct tparam *th, char *channel)
 
 	sprintf(data, "NAMES %s\r\n", channel);
 
-#ifdef DEBUG
-	printf("NAMES:sent data\n");
-#endif
+	DPRINTF(0, ("NAMES:sent data\n"));
+
 	send_postdata(th->socket_number, data);
 
 	return (0);
