@@ -1,6 +1,6 @@
-/* $Id: main.c,v 1.13 2005/12/16 17:55:25 ichiro Exp $ */
+/* $Id: main.c,v 1.27 2006/01/08 06:39:51 ichiro Exp $ */
 /*
- * Copyright (c) 2004
+ * Copyright (c) 2004, 2005, 2006
  *	Ichiro FUKUHARA <ichiro@ichiro.org>.
  * All rights reserved.
  *
@@ -46,6 +46,9 @@
 #include <sys/time.h>
 #include <sys/param.h>
 
+/* debug level (0 - 3)*/
+int debug = 2;
+
 #include "kircd.h"
 
 unsigned char *configfile = "kircd.conf";
@@ -150,15 +153,28 @@ int main(int argc, char *argv[])
 	th.l_buff = lbuff; 
 
 	/* copy config option */
-	th.max_line = cf->max_line;
+	if (cf->max_line > CHBUF_MAX)
+		th.max_line = CHBUF_MAX;
+	else
+		th.max_line = cf->max_line;
+
 	th.line_reverse = cf->line_reverse;
-	th.use_cookie = cf->use_cookie;
+	th.auth_method = cf->auth_method;
+	th.show_topic = cf->show_topic;
 	th.cookie_expire = cf->cookie_expire;
 	strcpy(th.real_nickname, cf->nickname);
 
-	/* copy auth info */
+	/* copy/init auth info */
 	strcpy(th.auth_user, cf->auth_user);
 	strcpy(th.auth_pass, cf->auth_pass);
+	strcpy(th.uid, "dummy");
+	th.auth_flag = 0;
+	th.regist_done = 0;
+	bzero(th.store_message, sizeof(th.store_message));
+
+	/* timer */
+	th.last_access = 0;
+	th.timeout = cf->timeout;
 
 	/* initialize memory allocate for send_buf */
 	th.send_buf = (char*)malloc(HTTPBUF * 2);
@@ -256,9 +272,7 @@ watch(gpointer param)
 		/* send data */
 		if (!th->t_empty) {
 			if (th->postdata[0] != '\0')
-#ifdef DEBUG
-			printf("send: %s\n",th->postdata);
-#endif
+			DPRINTF(0, ("send: %s\n",th->postdata));
 			/* SEND DATA */
 			send_postdata(s, th->postdata);
 			bzero(th->postdata, sizeof(th->postdata));
@@ -268,7 +282,7 @@ watch(gpointer param)
 
 		/* recieve data */
 		if (th->s_empty) {
-			bzero(sbuf, SBUFLEN);
+			bzero(sbuf, sizeof(sbuf));
 			th->s_count = 0;
 			th->s_read = 0;
 
@@ -291,7 +305,7 @@ lineget(gpointer param)
 	int lcount = 0;
 	th = (struct tparam *)param;
 
-	printf("keitairc ver %s\n", version);
+	printf("kircd ver %s\n", version);
 
 	do {
 		if (th->s_empty || th->s_lock) {
@@ -316,16 +330,18 @@ lineget(gpointer param)
 		/* bit empty flag */
 		if (th->s_count >= th->s_read)
 			th->s_empty = 1; 
-#ifdef DEBUG2
-		printf("(s_count %d/s_read %d/lcount %d)\n%s\n---------\n",
-			th->s_count, th->s_read, lcount, th->l_buff);
-		printf("s_empty=%d, s_lock=%d\n", th->s_empty, th->s_lock);
-		printf("s_read=%d, s_count=%d\n", th->s_read, th->s_count);
-#endif
+		DPRINTF(2, ("(s_count %d/s_read %d/lcount %d)\n%s\n"
+			    "---------\n",
+			    th->s_count, th->s_read, lcount, th->l_buff));
+		DPRINTF(2, ("s_empty=%d, s_lock=%d\n",
+			th->s_empty, th->s_lock));
+		DPRINTF(2, ("s_read=%d, s_count=%d\n",
+			th->s_read, th->s_count))
+;
 		keitai_start(th);
 
 		/* buffer clear */
-		bzero(lbuff, SBUFLEN);
+		bzero(lbuff, sizeof(lbuff));
 loop_end:
 	;;
 	} while (1);
